@@ -4,13 +4,10 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_API_V1_DBUS_DBUS_PEER_H_
-#define LINGLONG_API_V1_DBUS_DBUS_PEER_H_
+#pragma once
 
 #include "linglong/api/dbus/v1/gen_org_freedesktop_dbus_peer.h"
 
 namespace linglong::api::dbus::v1 {
 using DBusPeer = OrgFreedesktopDBusPeerInterface;
 }
-
-#endif
