@@ -47,7 +47,7 @@ void JSONPrinter::printReply(const api::types::v1::CommonResult &reply)
     std::cout << nlohmann::json(reply).dump() << std::endl;
 }
 
-void JSONPrinter::printRepoConfig(const api::types::v1::RepoConfig &config)
+void JSONPrinter::printRepoConfig(const api::types::v1::RepoConfigV2 &config)
 {
     std::cout << nlohmann::json(config).dump() << std::endl;
 }
@@ -83,4 +83,9 @@ void JSONPrinter::printUpgradeList(std::vector<api::types::v1::UpgradeListResult
     std::cout << nlohmann::json(list).dump(4) << std::endl;
 }
 
+void JSONPrinter::printInspect(const api::types::v1::InspectResult &result)
+{
+    std::cout << nlohmann::json(result).dump(4) << std::endl;
+}
+
 } // namespace linglong::cli
