@@ -14,7 +14,7 @@
 #define PACKAGE_INFO_VERSION "1.0"
 namespace linglong::utils {
 
-error::Result<api::types::v1::PackageInfoV2> toPackageInfoV2(const api::types::v1::PackageInfo &oldInfo);
+api::types::v1::PackageInfoV2 toPackageInfoV2(const api::types::v1::PackageInfo &oldInfo);
 error::Result<api::types::v1::PackageInfoV2> parsePackageInfo(const QString &path);
 error::Result<api::types::v1::PackageInfoV2> parsePackageInfo(const nlohmann::json &json);
 error::Result<api::types::v1::PackageInfoV2> parsePackageInfo(GFile *file);
